// ----------------------------------------------------------------------------
// This reference design and source code is being provided on an "as-is" basis 
// and as an accommodation, and therefore all warranties, representations or 
// guarantees of any kind (whether express, implied or statutory) including, 
// without limitation, warranties of merchantability, non-infringement, or  
// fitness for a particular purpose, are specifically disclaimed.
//
// This source code may only be used in an Altera programmable logic device
// and may not be distributed without permission from Macnica Americas, Inc.  
// It is provided free of royalties or fees of any kind.
// ----------------------------------------------------------------------------
//
// Description  | This module muxes signals from the HSMC from the Helio board
//              | and sends them to either an LCD display or HDMI TX device or
//              | Receives data from a HDMI TX device to send back to the 
//              | Helio board via the HSMC.  It also provides some GPIO.
//              | 
//              | 
//              | 
// Formatting   | Tabs set at 3 spaces.  Port names 'UPPER' case.
//              | Internal wires and registers are 'lower' case.
//              | Parameters are first character 'Upper' case.
//              | Active low signals are identified with '_n' or '_N'
//              | appended to the wire, register, or port name.
//              | 
//              | _r suffix on a register name indicates a simple
//              | re-registering of another wire or register.
//              | 
//              | _rr suffix on a register name indicates a second
//              | re-registering of a register.
//
// ----------------------------------------------------------------------------

//256 pin package - don't use G5

module LCD_board_cpld
(
	input                   clk               ,  //50 Mhz
	input                   rst               ,  //reset
  //HSMC Bank 1
  inout   wire            HSMA_SCL_3A       ,  //SCL                    I2C bus + CPLD      
  output  wire            HSMA_CLK_IN0_3B   ,  //HDMI_RX_PCLK           Out from CPLD       
  inout   wire            HSMA_SDA_3A       ,  //SDA                    I2C bus + CPLD      
  input   wire            HSMA_CLK_OUT_5B   ,  //HDMI_RX_PCLK           In to clk of CPLD  
  //HSMC Bank 2           
  inout   wire            HSMA_D1_4A        ,  //Int/Rst/PWM/Misc/Sel   CPLD GPIO           
  inout   wire            HSMA_D3_4A        ,  //Int/Rst/PWM/Misc/Sel   CPLD GPIO           
  inout   wire            HSMA_RX_D_P0_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_M0_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_P1_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_M1_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_P2_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_M2_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_P3_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_M3_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_P4_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_M4_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_P5_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_M5_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_P6_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_M6_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_P7_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_RX_D_M7_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_D0_4A        ,  //Int/Rst/PWM/Misc/Sel   CPLD GPIO           
  inout   wire            HSMA_D2_4A        ,  //Int/Rst/PWM/Misc/Sel   CPLD GPIO           
  inout   wire            HSMA_TX_D_P0_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_TX_D_M0_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_TX_D_P1_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_TX_D_M1_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_TX_D_P2_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_TX_D_M2_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_TX_D_P3_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_TX_D_M3_4A   ,  //LCD COLOR / HDMI TX    CPLD GPIO           
  inout   wire            HSMA_TX_D_P4_4A   ,  //LCD SYNCS              CPLD GPIO           
  inout   wire            HSMA_TX_D_M4_4A   ,  //LCD SYNCS              CPLD GPIO           
  inout   wire            HSMA_TX_D_P5_4A   ,  //LCD SYNCS              CPLD GPIO           
  inout   wire            HSMA_TX_D_M5_4A   ,  //rx_hdmi_enable         CPLD GPIO           
  inout   wire            HSMA_TX_D_P6_4A   ,  //RSTn_SOC_RX            CPLD GPIO           
  inout   wire            HSMA_TX_D_M6_4A   ,  //RSTn_SOC_TP            CPLD GPIO 
  //HSMC Bank 3            
  inout   wire            HSMA_RX_D_P8_3B   ,  //HDMI RX COLOR / EXP2   CPLD GPIO           
  inout   wire            HSMA_RX_D_M8_3B   ,  //HDMI RX COLOR / EXP2   CPLD GPIO           
  inout   wire            HSMA_RX_D_P9_3B   ,  //HDMI RX COLOR / EXP2   CPLD GPIO           
  inout   wire            HSMA_RX_D_M9_3B   ,  //HDMI RX COLOR / EXP2   CPLD GPIO           
  inout   wire            HSMA_RX_D_P11_3B  ,  //HDMI RX COLOR / EXP2   CPLD GPIO           
  inout   wire            HSMA_RX_D_M11_3B  ,  //HDMI RX COLOR / EXP2   CPLD GPIO           
  inout   wire            HSMA_RX_D_P12_3B  ,  //HDMI RX COLOR / EXP2   CPLD GPIO           
  inout   wire            HSMA_RX_D_M12_3B  ,  //HDMI RX COLOR / EXP2   CPLD GPIO           
  inout   wire            HSMA_RX_D_P13_3B  ,  //HDMI RX COLOR / EXP2   CPLD GPIO           
  inout   wire            HSMA_RX_D_M13_3B  ,  //HDMI RX COLOR / EXP2   CPLD GPIO           
  inout   wire            HSMA_RX_D_P16_3B  ,  //HDMI RX COLOR / EXP2   CPLD GPIO           
  inout   wire            HSMA_RX_D_M16_3B  ,  //HDMI RX COLOR / EXP2   CPLD GPIO           
  inout   wire            HSMA_TX_D_P8_4A   ,  //HDMI RX / EXP2         CPLD GPIO           
  inout   wire            HSMA_TX_D_M8_4A   ,  //HDMI RX / EXP1         CPLD GPIO           
  inout   wire            HSMA_TX_D_P9_4A   ,  //HDMI RX / EXP1         CPLD GPIO           
  inout   wire            HSMA_TX_D_M9_4A   ,  //HDMI RX / EXP1         CPLD GPIO           
  inout   wire            HSMA_TX_D_P10_3B  ,  //HDMI RX COLOR / EXP1   CPLD GPIO           
  inout   wire            HSMA_TX_D_M10_3B  ,  //HDMI RX COLOR / EXP1   CPLD GPIO           
  inout   wire            HSMA_TX_D_P11_3B  ,  //HDMI RX COLOR / EXP1   CPLD GPIO           
  inout   wire            HSMA_TX_D_M11_3B  ,  //HDMI RX COLOR / EXP1   CPLD GPIO           
  inout   wire            HSMA_TX_D_P12_3B  ,  //HDMI RX SYNC/ EXP1     CPLD GPIO           
  inout   wire            HSMA_TX_D_M12_3B  ,  //HDMI RX SYNC/ EXP1     CPLD GPIO           
  inout   wire            HSMA_TX_D_P13_3B  ,  //HDMI RX SYNC/ EXP1     CPLD GPIO           
  inout   wire            HSMA_TX_D_M13_3B  ,  //EXP1/misc              CPLD GPIO           
	//LCD Panel (all 3.3V signals)
	output  wire  [7:0]     VID_DISP_R        ,
	output  wire  [7:0]     VID_DISP_G        ,
	output  wire  [7:0]     VID_DISP_B        ,
	output  wire            VID_DISP_HS       ,
	output  wire            VID_DISP_VS       ,
	output  wire            VID_DISP_DE       ,
	output  wire            VID_DISP_PCLK     ,
	output  wire            STBYn_DISP        ,
	output  wire            PWM_SOC_LED       ,
	input   wire            TP_INT            ,
	output  wire            RSTn_SOC_TP       ,
	//HDMI TX
	output  wire  [7:0]     VID_TX_CBCR      ,
	output  wire  [7:0]     VID_TX_Y         ,
	output  wire            VID_TX_HX        ,
	output  wire            VID_TX_VS        ,
	output  wire            VID_TX_DE        ,
	output  wire            VID_TX_PCLK      ,
	output  wire  [3:0]     AUD_TX_SD        ,
	output  wire            AUD_TX_WS        ,
	output  wire            AUD_TX_ACLK      ,
	input   wire            INTn_TX           ,
	//HDMI RX
	input   wire  [7:0]     VID_RX_CBCR      ,
	input   wire  [7:0]     VID_RX_Y         ,
	input   wire            VID_RX_HX        ,
	input   wire            VID_RX_VS        ,
	input   wire            VID_RX_DE        ,
	input   wire            VID_RX_PCLK      ,
	input   wire  [3:0]     AUD_RX_SD        ,
	input   wire            AUD_RX_WS        ,
	input   wire            AUD_RX_ACLK      ,
	input   wire            INTn_RX           ,
	output  wire            RSTn_SOC_RX       ,
	//Expansion connector 1
	inout   wire  [10:0]    EX1_GPIO          ,
	input   wire            EX1_CLK           ,
	//Expansion connector 2
	inout   wire  [12:0]    EX2_GPIO          ,
	input   wire            EX2_CLK           ,
	//test
	output  reg   [3:0]     TP                ,
	//misc
	input	wire  [2:1]       cpld_sw           ,
	output  wire  [3:0]     cpld_led          ,
	input   wire            power_good

);

wire           lcd_enable;
wire           rx_hdmi_enable;

//
// simple connections for SW inputs and LED outputs
//
assign cpld_led[1:0] = cpld_sw[2:1];
assign cpld_led[2]   = lcd_enable;
assign cpld_led[3]   = rx_hdmi_enable;



//
// Create a Bus to simplify HSMC connections
//
wire [27:0] hsmc_tx_vid;
assign hsmc_tx_vid = {HSMA_RX_D_P0_4A,
                      HSMA_RX_D_M0_4A,
                      HSMA_RX_D_P1_4A,
                      HSMA_RX_D_M1_4A,
                      HSMA_RX_D_P2_4A,
                      HSMA_RX_D_M2_4A,
                      HSMA_RX_D_P3_4A,
                      HSMA_RX_D_M3_4A,
                      HSMA_RX_D_P4_4A,
                      HSMA_RX_D_M4_4A,
                      HSMA_RX_D_P5_4A,
                      HSMA_RX_D_M5_4A,
                      HSMA_RX_D_P6_4A,
                      HSMA_RX_D_M6_4A,
                      HSMA_RX_D_P7_4A,
                      HSMA_RX_D_M7_4A,
                      HSMA_TX_D_P0_4A,
                      HSMA_TX_D_M0_4A,
                      HSMA_TX_D_P1_4A,
                      HSMA_TX_D_M1_4A,
                      HSMA_TX_D_P2_4A,
                      HSMA_TX_D_M2_4A,
                      HSMA_TX_D_P3_4A,
                      HSMA_TX_D_M3_4A,
                      HSMA_TX_D_P4_4A,
                      HSMA_TX_D_M4_4A,
                      HSMA_TX_D_P5_4A,
                      1'b1};      
                     
//
// Mux for LCD output
//
	assign    VID_DISP_R    = lcd_enable ? hsmc_tx_vid[27:20] : 8'h00; 
	assign    VID_DISP_G    = lcd_enable ? hsmc_tx_vid[19:12] : 8'h00;
	assign    VID_DISP_B    = lcd_enable ? hsmc_tx_vid[11:4]  : 8'h00;
	assign    VID_DISP_HS   = lcd_enable ? hsmc_tx_vid[3]     : 1'b0;
	assign    VID_DISP_VS   = lcd_enable ? hsmc_tx_vid[2]     : 1'b0;
	assign    VID_DISP_DE   = lcd_enable ? hsmc_tx_vid[1]     : 1'b0;
	assign    PWM_SOC_LED   = lcd_enable ? hsmc_tx_vid[0]     : 1'b0;

//
// Mux for HDMI TX output
//
	assign    VID_TX_CBCR  = lcd_enable ? 8'h00 : hsmc_tx_vid[19:12];  
	assign    VID_TX_Y     = lcd_enable ? 8'h00 : hsmc_tx_vid[27:20];
	assign    VID_TX_HX    = lcd_enable ? 1'b0  : hsmc_tx_vid[3]    ;
	assign    VID_TX_VS    = lcd_enable ? 1'b0  : hsmc_tx_vid[2]    ; 
	assign    VID_TX_DE    = lcd_enable ? 1'b0  : hsmc_tx_vid[1]    ; 
	assign    AUD_TX_SD    = lcd_enable ? 4'h0  : hsmc_tx_vid[9:6]  ;
	assign    AUD_TX_WS    = lcd_enable ? 1'b0  : hsmc_tx_vid[5]    ;
	assign    AUD_TX_ACLK  = lcd_enable ? 1'b0  : hsmc_tx_vid[4]    ;
//	assign    INTn_TX       = lcd_enable ? 1'b0  : hsmc_tx_vid[0]    ;
	//hsmc_tx_vid[11:10] and hsmc_tx_vid[0] unassigned in HDMI mode

//
// Create a Bus to simplify HSMC connections
//
wire [19:0] 	hsmc_rx_vid;
	assign 				 {HSMA_RX_D_P8_3B   ,
	                    HSMA_RX_D_M8_3B   ,
	                    HSMA_RX_D_P9_3B   ,
	                    HSMA_RX_D_M9_3B   ,
	                    HSMA_RX_D_P11_3B  ,
	                    HSMA_RX_D_M11_3B  ,
	                    HSMA_RX_D_P12_3B  ,
	                    HSMA_RX_D_M12_3B  ,
	                    HSMA_RX_D_P13_3B  ,
	                    HSMA_RX_D_M13_3B  ,
	                    HSMA_RX_D_P16_3B  ,
	                    HSMA_RX_D_M16_3B  ,
	                    HSMA_TX_D_P10_3B  ,
	                    HSMA_TX_D_M10_3B  ,
	                    HSMA_TX_D_P11_3B  ,
	                    HSMA_TX_D_M11_3B  ,
	                    HSMA_TX_D_P12_3B  ,
	                    HSMA_TX_D_M12_3B  ,
	                    HSMA_TX_D_P13_3B  ,
	                    HSMA_TX_D_M13_3B  } = hsmc_rx_vid;

//
// Create a Bus to simplify HSMC connections
//
	wire [3:0] hsmc_rx_aud;
	assign             {HSMA_TX_D_P8_4A   ,
	                    HSMA_TX_D_M8_4A   ,
	                    HSMA_TX_D_P9_4A   ,
	                    HSMA_TX_D_M9_4A   } = hsmc_rx_aud;
	                    
	
	assign VID_DISP_PCLK = lcd_enable ? HSMA_CLK_OUT_5B : 1'b0;
	assign VID_TX_PCLK  = lcd_enable ? 1'b0 : HSMA_CLK_OUT_5B;
	
	assign STBYn_DISP    = lcd_enable;
	assign HSMA_CLK_IN0_3B = rx_hdmi_enable ? VID_RX_PCLK : EX2_CLK;

//
// mux for HDMI RX video data or GPIO
//
	assign hsmc_rx_vid = rx_hdmi_enable ? {VID_RX_CBCR, VID_RX_Y, VID_RX_HX, VID_RX_VS, VID_RX_DE, 1'b0} : 
                        {EX2_GPIO[12:1], EX1_GPIO[7:0]};

//
// mux for HDMI RX audio data or GPIO
//
	assign hsmc_rx_aud = rx_hdmi_enable ? {AUD_RX_SD[1:0], AUD_RX_WS, AUD_RX_ACLK} : 
                        {EX2_GPIO[0], EX1_GPIO[10:8]};

	assign HSMA_D1_4A = INTn_RX;
	assign HSMA_D0_4A = TP_INT;
	assign HSMA_D3_4A = INTn_TX;
	assign RSTn_SOC_RX = cpld_sw[1]; //map allow hdmi rx to be reset by switch
	assign RSTn_SOC_TP = cpld_sw[1]; //HSMA_TX_D_M6_4A;
	assign rx_hdmi_enable = 1'b1; //HSMA_TX_D_M5_4A;
  

debounce u_debounce
(
	.reset (rst),
	.clk   (clk),
	.noisy (cpld_sw[2]),
	.clean (lcd_enable)
);

assign HSMA_D2_4A = lcd_enable;
	
endmodule
